package cat.hack3.codingtests.marsrover.test;

import cat.hack3.codingtests.marsrover.api.RotableRiderRover;
import cat.hack3.codingtests.marsrover.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.cartography.Direction;
import cat.hack3.codingtests.marsrover.cartography.PlanetMap;
import cat.hack3.codingtests.marsrover.commands.api.CommandFactory;
import cat.hack3.codingtests.marsrover.commands.api.RoverCommand;
import cat.hack3.codingtests.marsrover.plugin.station.PlanetMapInitializerProvider;
import cat.hack3.codingtests.marsrover.plugin.station.RoverDeployerProvider;

import java.util.List;
import java.util.stream.IntStream;

import static cat.hack3.codingtests.marsrover.cartography.Direction.SOUTH;
import static cat.hack3.codingtests.marsrover.commands.api.RoverCommand.Type.*;

public class CommonTests {

    public static final Direction STARTING_DIRECTION = SOUTH;

    protected RotableRiderRover rover;
    protected RoverCommand moveForwardCommand;
    protected RoverCommand moveBackwardsCommand;
    protected RoverCommand turnLeftCommand;
    protected RoverCommand turnRightCommand;

    private PlanetMapInitializerProvider gpsProvider;
    private RoverDeployerProvider roverProvider;

    protected void setup() {
        gpsProvider = new PlanetMapInitializerProvider();
        roverProvider = new RoverDeployerProvider();

        int mapWidth = 10;
        int mapHeight = 10;
        int latitudeStartingPoint = 2;
        int longitudeStartingPoint = 3;
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);

        var marsMap = getMap(mapHeight, mapWidth, startingCoordinates);
        rover = getRover(marsMap);

        var commandFactory = new CommandFactory(this.rover);
        moveForwardCommand = commandFactory.createCommand(MOVE_FORWARD);
        moveBackwardsCommand = commandFactory.createCommand(MOVE_BACKWARDS);
        turnLeftCommand = commandFactory.createCommand(TURN_LEFT);
        turnRightCommand = commandFactory.createCommand(TURN_RIGHT);
    }

    private PlanetMap getMap(int mapHeight, int mapWidth, Coordinates startingCoordinates) {
        var planetInitializer = gpsProvider.getNextAvailable();
        return planetInitializer.setupNewMap(mapHeight, mapWidth, startingCoordinates, getObstacles());
    }

    private RotableRiderRover getRover(PlanetMap marsMap) {
        var roverDeployer = roverProvider.getNextAvailable();
        return roverDeployer.deploy(marsMap, CommonTests.STARTING_DIRECTION);
    }

    protected List<Coordinates> getObstacles() {
        return List.of();
    }

    protected void repeatAction(int times, Runnable actionToRepeat) {
        IntStream.rangeClosed(1, times)
                .forEach(i -> actionToRepeat.run());
    }
}
